/*
 
 Copyright (C) 2006 NTT DATA Corporation
 
 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation, version 2.
 
 This program is distributed in the hope that it will be
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 */

package com.clustercontrol.monitor.run.composite;

import java.util.ArrayList;

import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;

import com.clustercontrol.bean.ProcessConstant;
import com.clustercontrol.dialog.ValidateResult;
import com.clustercontrol.monitor.run.bean.MonitorInfo;
import com.clustercontrol.monitor.run.bean.MonitorStringValueInfo;
import com.clustercontrol.monitor.run.util.StringValueInfoManager;
import com.clustercontrol.util.Messages;

/**
 * 文字列監視判定 一覧コンポジットクラスです。
 * 
 * @version 2.1.0
 * @since 2.1.0
 */
public class StringValueListComposite extends Composite {
	
	// ----- instance フィールド ----- //
	
	/** テーブルビューア */
	private StringValueListTableViewer m_tableViewer = null;
	
	/** テーブル定義情報 */
	private ArrayList m_tableDefine = null;
	
	/** 通知ID設定フラグ */
	private boolean m_notify = false;
	
	// ----- コンストラクタ ----- //
	
	/**
	 * インスタンスを返します。
	 * 
	 * @param parent 親のコンポジット
	 * @param style スタイル
	 */
	public StringValueListComposite(Composite parent, int style) {
		super(parent, style);
		
		this.initialize();
	}
	
	/**
	 * インスタンスを返します。
	 * 
	 * @param parent 親のコンポジット
	 * @param style スタイル
	 * @param tableDefine テーブル定義
	 */
	public StringValueListComposite(Composite parent, int style, ArrayList tableDefine) {
		super(parent, style);
		
		this.m_tableDefine = tableDefine;
		
		this.initialize();
	}
	
	// ----- instance メソッド ----- //
	
	/**
	 * コンポジットを生成・構築します。
	 */
	private void initialize() {
		GridLayout layout = new GridLayout(1, true);
		this.setLayout(layout);
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		
		Table table = new Table(this, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		table.setLayoutData(gridData);
		
		// テーブルビューアの作成
		this.m_tableViewer = new StringValueListTableViewer(table);
		this.m_tableViewer.createTableColumn(this.m_tableDefine);
		
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
	}
	
	/**
	 * tableViewerを返します。
	 * 
	 * @return tableViewer
	 */
	public StringValueListTableViewer getTableViewer() {
		return this.m_tableViewer;
	}
	
	/**
	 * 現在選択されているアイテムを返します。
	 * <p>
	 * 
	 * 選択されていない場合は、nullを返します。
	 * 
	 * @return 選択アイテム
	 */
	public MonitorStringValueInfo getFilterItem() {
		StructuredSelection selection = (StructuredSelection) this.m_tableViewer.getSelection();
		
		if (selection == null) {
			return null;
		} else {
			return (MonitorStringValueInfo) selection.getFirstElement();
		}
	}
	
	/**
     * 各項目に入力値を設定します。
     * 
     * @param info 設定値として用いる監視情報
     */
	public void setInputData(MonitorInfo monitorInfo) {
		
		if(monitorInfo != null){
			// 文字列監視判定情報設定
			StringValueInfoManager.getInstance().initialize(monitorInfo);
			// データ取得
			Object[] list = StringValueInfoManager.getInstance().get();
			
			// テーブル更新
			this.m_tableViewer.setInput(list);
		}
	}
	
	/**
     * コンポジットを更新します。
     * 
     */
    public void update() {
        // データ取得
        Object[] list = StringValueInfoManager.getInstance().get();

        // テーブル更新
        this.m_tableViewer.setInput(list);
    }
	
	/**
     * 入力値を用いて監視情報を生成します。
     * 
     * @return 検証結果
     */
	public ValidateResult createInputData(MonitorInfo monitorInfo) {
		
		m_notify = false;
		ArrayList list = StringValueInfoManager.getInstance().getMonitorStringValueInfo();

		if(list != null && list.size() > 0){
			String MonitorTypeId = monitorInfo.getMonitorTypeId();
			String MonitorId = monitorInfo.getMonitorId();
			
			for(int index=0; index<list.size(); index++){
				MonitorStringValueInfo info = (MonitorStringValueInfo)list.get(index);
				info.setMonitorTypeId(MonitorTypeId);
				info.setMonitorId(MonitorId);
				
				if(info.getNotifyId() != null && !"".equals(info.getNotifyId())){
					if(info.getProcessType() == ProcessConstant.TYPE_YES){
						m_notify = true;
					}
				}
			}
			monitorInfo.setJudgementInfo(list);
		}
		else{
			return setValidateResult(Messages.getString("message.hinemos.1"),Messages.getString("message.monitor.32"));
		}
		return null;
	}
	
	/**
     * 無効な入力値の情報を設定します
     *  
     */
    protected ValidateResult setValidateResult(String id, String message) {

    	ValidateResult validateResult = new ValidateResult();
        validateResult.setValid(false);
        validateResult.setID(id);
        validateResult.setMessage(message);
        
        return validateResult;
    }

	/**
	 * 通知IDが設定されているかを返します。
	 * 
	 * @return 1件でも通知IDが設定されている場合は、true。
	 */
	public boolean isNotify() {
		return m_notify;
	}
}