/*
 
Copyright (C) 2006 NTT DATA Corporation
 
This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation, version 2.
 
This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.
 
*/

package com.clustercontrol.jobmanagement.bean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.clustercontrol.bean.JobConstant;

/**
 * ジョブツリーアイテムアイテム
 * ジョブのツリー表示に関する情報を格納するクラスです。
 * 
 * @version 2.0.0
 * @since 1.0.0
 */
public class JobTreeItem implements Serializable, Cloneable {
	/** シリアライズ可能クラスに定義するUID */
	private static final long serialVersionUID = -5749478055659165471L;

	/** ジョブ情報 */
	protected JobInfo m_data;

	/** 子のジョブツリーアイテムのリスト */
    protected List<JobTreeItem> m_children = new ArrayList<JobTreeItem>();

    /** 親のジョブツリーアイテム */
    protected JobTreeItem m_parent;

    /** パスセパレータ */
    public static final String SEPARATOR = ">";

    public JobTreeItem() {
    }

    /**
     * コンストラクタ
     * ジョブツリーにジョブ情報を紐付ける
     * 
     * @param data ジョブ情報
     * @see com.clustercontrol.jobmanagement.bean.JobInfo
     */
    public JobTreeItem(JobInfo data) {
        setData(data);
    }

    /**
     * コンストラクタ
     * ジョブツリーに親ジョブツリーを紐付ける
     * ジョブツリーにジョブ情報を紐付ける
     * 
     * @param parent 親のジョブツリーアイテム
     * @param data ジョブ情報
     * @see com.clustercontrol.jobmanagement.bean.JobInfo
     */
    public JobTreeItem(JobTreeItem parent, JobInfo data) {

        this.setParent(parent);
        this.setData(data);

        if (parent != null) {
            parent.addChildren(this);
        }

        this.m_children = new ArrayList<JobTreeItem>();
    }

    /**
     * 親のジョブツリーアイテムを設定する
     * @param parent 親のジョブツリーアイテム
     */
    public void setParent(JobTreeItem parent) {
        this.m_parent = parent;
    }

    /**
     * 親のジョブツリーアイテムを返す
     * @return 親のジョブツリーアイテム
     */
    public JobTreeItem getParent() {
        return m_parent;
    }

    /**
     * ジョブ情報を設定する
     * @param data ジョブ情報
     */
    public void setData(JobInfo data) {
        this.m_data = data;
    }

    /**
     * ジョブ情報を返す
     * @return ジョブ情報
     */
    public JobInfo getData() {
        return m_data;
    }

    /**
     * 子のジョブツリーアイテムを、子のジョブツリーアイテムのリストに追加する
     * @param child 子のジョブツリーアイテム
     */
    public void addChildren(JobTreeItem child) {
        m_children.add(child);
        child.setParent(this);
    }

    /**
     * 子のジョブツリーアイテムを、子のジョブツリーアイテムのリストから削除する
     * @param child 子のジョブツリーアイテム
     */
    public void removeChildren(JobTreeItem child) {
        for (int i = 0; i < m_children.size(); i++) {
            if (child.equals(m_children.get(i))) {
                m_children.remove(i);
                break;
            }
        }
    }

    /**
     * 子のジョブツリーアイテムの数を返す
     * @return 子のジョブツリーアイテムの数
     */
    public int size() {
        return m_children.size();
    }

    /**
     * 子のジョブツリーアイテムを返す
     * @param index 子のジョブツリーアイテムのリストインデックス
     * @return 子のジョブツリーアイテム
     */
    public JobTreeItem getChildren(int index) {
        return (JobTreeItem) m_children.get(index);
    }

    /**
     * 子のジョブツリーアイテムの配列を返す
     * @return 子のジョブツリーアイテムの配列
     */
    public JobTreeItem[] getChildren() {
        return (JobTreeItem[])m_children.toArray(new JobTreeItem[m_children.size()]);
    }

    /**
     * ジョブツリーアイテムの親子関係を表現するパス文字列を返す。
     * <p>
     * 例）以下のジョブツリーにて、getPath()を呼び出す
     * <p>
	 * <ul>
	 *  <li>料金システム
	 *  <ul>
	 *   <li>顧客管理
	 *   <ul>
	 *    <li>WEB  <- このインスタンスにてgetPath()を呼び出す
	 *    <li>DB
	 *   </ul>
	 *  </ul>
	 * </ul>
     * <p>
     * 結果 ： "料金システム>顧客管理>WEB"
     * 
     * @return パス文字列
     */
    public String getPath() {

        // トップ("ジョブ")の場合は、文字を出力しません。
        if (this.getData().getType() == JobConstant.TYPE_COMPOSITE) {
            return "";
        }

        StringBuffer buffer = new StringBuffer();

        buffer.append(this.getData().getName());

        /*
         * 再起呼び出しすることでもないので。
         */

        JobTreeItem parent = this.getParent();
        while (parent != null
                && parent.getData().getType() != JobConstant.TYPE_COMPOSITE) {

            buffer.insert(0, SEPARATOR);
            buffer.insert(0, parent.getData().getName());
            parent = parent.getParent();
        }

        return buffer.toString();
    }
    
	/**
	 * ジョブツリーアイテムのクローンを作成する
	 * 
	 * @see java.lang.Object#clone()
	 */
    public Object clone() {
        try {
            JobTreeItem clone = (JobTreeItem)super.clone();
            
            clone.setData((JobInfo)this.getData().clone());

            clone.m_children = new ArrayList<JobTreeItem>();
	        if(this.m_children != null && this.m_children instanceof ArrayList){
	            for(int i = 0; i < this.m_children.size(); i++){
	                Object source = this.m_children.get(i);
	                if(source != null && source instanceof JobTreeItem){
	                    clone.addChildren((JobTreeItem)((JobTreeItem)source).clone());
	                }
	            }
            }
            
            return clone;
        } catch (CloneNotSupportedException e) {
            return null;
        }
    }
}