/*
 
 Copyright (C) 2006 NTT DATA Corporation
 
 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation, version 2.
 
 This program is distributed in the hope that it will be
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 */

package com.clustercontrol.process.factory;

import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.jms.JMSException;
import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.bean.PriorityConstant;
import com.clustercontrol.monitor.run.factory.RunMonitorNumericValueType;
import com.clustercontrol.process.bean.SnmpSharedTableConstant;
import com.clustercontrol.process.ejb.entity.MonitorProcessInfoLocal;
import com.clustercontrol.process.ejb.entity.MonitorProcessInfoPK;
import com.clustercontrol.process.ejb.entity.MonitorProcessInfoUtil;
import com.clustercontrol.process.util.ProcessProperties;
import com.clustercontrol.process.util.SnmpProcConnectionManager;
import com.clustercontrol.sharedtable.bean.ValueObject;
import com.clustercontrol.snmppoller.SnmpSharedTable;
import com.clustercontrol.util.Messages;

/**
 * プロセス監視クラス
 *
 * @version 2.0.0
 * @since 2.0.0
 */
public class RunMonitorProcess extends RunMonitorNumericValueType {
	
	protected static Log m_log = LogFactory.getLog( RunMonitorProcess.class );
	
	public static final String MESSAGE_ID_INFO = "001";
	public static final String MESSAGE_ID_WARNING = "002";
	public static final String MESSAGE_ID_CRITICAL = "003";
	public static final String MESSAGE_ID_UNKNOWN = "004";
	
	/** プロセス監視情報 */
	protected MonitorProcessInfoLocal m_process = null;
	
	/** コマンド */
	protected String m_command = "";
	
	/** 引数 */
	protected String m_param = "";
	
	/** SNMP収集値の共有テーブル */
	protected SnmpSharedTable m_sst = null;
	
	/** メッセージ */
	protected String m_message = null;
	
	/** オリジナルメッセージ */
	protected String m_messageOrg = null;
	
	/** 数値フォーマット */
	protected NumberFormat m_numberFormat = NumberFormat.getNumberInstance();
	
	
	/**
	 * コンストラクタ
	 * 
	 */
	public RunMonitorProcess() throws NamingException, JMSException, CreateException{
		super();
		
		// SNMP収集値の共有テーブルをルックアップ
		setSnmpSharedTable();
	}
	
	/**
	 * プロセス数を取得
	 * 
	 * @param facilityId ファシリティID
	 * @return 値取得に成功した場合、true
	 */
	@Override
	@SuppressWarnings("unchecked")
	public boolean collect(String facilityId) {
		
		// 引数のコマンド,パラメータに一致したプロセス数
		int count = 0;
		
		// 監視開始時刻を設定
		m_nodeDate = m_now.getTime();
		m_value = 0;
		
		// メッセージを設定
		m_message = "";
		m_messageOrg = Messages.getString("command") + " : " + m_command + ", " + Messages.getString("param") + " : " + m_param;
		
		// コマンド,パラメータのパターンを取得
		Pattern pCommand = null;
		Pattern pParam = null;
		Matcher m = null;
		
		// 収得時間の最小値を格納
		long mostOldPollingTime = Long.MAX_VALUE;
		
		try{
			pCommand = Pattern.compile(m_command);
			pParam = Pattern.compile(m_param);
		}
		catch(PatternSyntaxException e){
			m_log.error("collect(): コマンド,パラメータ正規表現パターン構文エラー。" + e.getMessage());
			
			m_message = Messages.getString("message.process.4");
			return false;
		}
		
		// 実行間隔
		int runInterval = m_monitor.getRunInterval().intValue();
		
		// 起動名を指定してサブツリーの値を一度に取得
		Set valueSetName = m_sst.getValueSet(facilityId, runInterval, SnmpSharedTableConstant.OID_RUN_NAME);
		// 起動パラメータのOIDを指定してサブツリーの値を一度に取得
		Set valueSetParam = m_sst.getValueSet(facilityId, runInterval, SnmpSharedTableConstant.OID_RUN_PARAM);
		
		
		if(valueSetName == null || valueSetParam == null){
			m_log.debug("collect(): " + "SNMPポーラー値未収集. valueSetName is null.");
			
			m_message = Messages.getString("message.process.6");
			return false;
		} else {
			
			ValueObject valueObjCommand = null;
			ValueObject valueObjParam = null;
			ValueObject valueObjName = null;
			String valueCommand = null;
			String valueParam = null;
			String valueName = null;
			
			Iterator itr = valueSetName.iterator();
			while(itr.hasNext()){
				valueObjName = (ValueObject)itr.next();
				
				// 取得時間がmostOldPollingTimeより小さい場合は、置き換える
				if(mostOldPollingTime > valueObjName.getDate())
					mostOldPollingTime = valueObjName.getDate();
				
				if(valueObjName != null){
					valueName = (String)valueObjName.getValue();
					if(valueName == null){
						m_log.debug("collect(): " + "SNMPポーラー値未収集. valueName is null.");
						m_message = Messages.getString("message.process.6");
						return false;
					}	
				}
				else{
					m_log.debug("collect(): " + "SNMPポーラー値未収集. valueObjName is null.");
					m_message = Messages.getString("message.process.6");
					return false;
				}
				
				// インデックス付きの起動パスOIDを取得
				String key = valueObjName.getKey();
				String index = key.substring(key.lastIndexOf("."));
				
				// 起動パスOIDを指定して値を取得
				valueObjCommand  = m_sst.getValue(facilityId, runInterval, SnmpSharedTableConstant.OID_RUN_PATH + index);
				if(valueObjCommand != null){
					valueCommand = (String)valueObjCommand.getValue();
					if(valueCommand == null){
						m_log.debug("collect(): " + "SNMPポーラー値未収集. valueCommand is null");
						m_message = Messages.getString("message.process.6");
						return false;
					}
				}
				else{
					m_log.debug("collect(): " + "SNMPポーラー値未収集. valueObjCommand is null");
					m_message = Messages.getString("message.process.6");
					return false;
				}
				
				//Windowsの場合を考慮して、valueCommandとvalueNameを連結する
				if(valueCommand.length() == 0){
					//パスが取得できない場合はコマンド名
					//パスがnull OR 空文字
					valueCommand = valueName;
				}else if(!valueCommand.startsWith("/") && 
						valueCommand.endsWith("\\") &&
						!valueCommand.equals(valueName) ){
					//条件・・・・
					//パスが'/'以外の文字で始まり
					//パスが'\'で終わっていて
					//パスとコマンド名が違う
					
					valueCommand = valueCommand + valueName;
				}
				
				
				m = pCommand.matcher(valueCommand);
				
				// 起動パスが、指定したコマンドを一致する場合
				if(m.matches()) {
					m_log.debug("collect(): SNMPポーラ収集起動パス: " + valueObjCommand);
					
					
					// 起動パラメータOIDを指定して値を取得
					valueObjParam = m_sst.getValue(facilityId, runInterval, SnmpSharedTableConstant.OID_RUN_PARAM + index);
					m_log.debug("collect(): facilityID : " + facilityId.toString() + " valueObjParam: " + valueObjParam);
					
					if(valueObjParam != null){
						valueParam = (String)valueObjParam.getValue();
						if(valueParam == null){
							m_log.debug("collect(): " + "SNMPポーラー値未収集. valueParam is null.");
							m_message = Messages.getString("message.process.6");
							return false;
						}	
					}
					else{
						m_log.debug("collect(): " + "SNMPポーラー値未収集. valueObjParam is null.");
						m_message = Messages.getString("message.process.6");
						return false;
					}
					
					//起動パラメータが、指定した引数と一致する場合
					m = pParam.matcher(valueParam);
					if (m.matches()) {
						m_log.debug("collect(): SNMPポーラ収集起動パラメータ : " + valueObjParam);
						
						// SNMPポーラ収集時刻を取得
						Date pollingDate = new Date(valueObjCommand.getDate()); 
						
						// 収集時刻がSNMPポーラー収集許容時間よりも前だった場合、値取得失敗
						int tolerance = ProcessProperties.getProperties().getStartSecond() + 
						ProcessProperties.getProperties().getValidSecond();
						
						Calendar cal = Calendar.getInstance();
						cal.setTime(m_now);
						cal.add(Calendar.SECOND, -tolerance);
						if(cal.getTime().compareTo(pollingDate) > 0){
							// 監視開始時刻を設定
							m_nodeDate = m_now.getTime();
							
							String[] args = { DateFormat.getDateTimeInstance().format(pollingDate)};
							m_message = Messages.getString("message.process.7",args);
							return false;
						}
						
						// ノードの値取得時刻を設定
						m_nodeDate = valueObjCommand.getDate();
						count++;
					}
				}
			}
			
			// カウントが0の場合の収集時刻判定
			// 取得された最も古い時間と現在の時間を比較する
			if (count == 0){
				
				// 収集時刻がSNMPポーラー収集許容時間よりも前だった場合、値取得失敗
				long tolerance = ( ProcessProperties.getProperties().getStartSecond() +
						ProcessProperties.getProperties().getValidSecond() ) * 1000;
				
				if(m_now.getTime() - mostOldPollingTime - (long)tolerance> 0){
					//      監視開始時刻を設定
					m_nodeDate = m_now.getTime();
					
					Date mostOldPollingDate = new Date(mostOldPollingTime);
					
					String[] args = { DateFormat.getDateTimeInstance().format(mostOldPollingDate)};
					m_message = Messages.getString("message.process.16",args);
					return false;
				}
			}
			
			m_value = count;
			m_message = Messages.getString("process.number") + " : " + m_numberFormat.format(m_value);
			return true;
		}
	}
	
	/* (非 Javadoc)
	 * プロセス監視情報を設定
	 * @see com.clustercontrol.monitor.run.factory.OperationNumericValueInfo#setMonitorAdditionInfo()
	 */
	@Override
	public void setCheckInfo() throws FinderException, NamingException{
		
		// プロセス監視情報を取得
		MonitorProcessInfoPK pk = new MonitorProcessInfoPK(m_monitorId, m_monitorTypeId);
		m_process = MonitorProcessInfoUtil.getLocalHome().findByPrimaryKey(pk);
		
		// プロセス監視情報を設定
		m_command = m_process.getCommand().trim();
		if(m_process.getParam() != null){
			m_param = m_process.getParam().trim();
		}
	}
	
	/* (非 Javadoc)
	 * ノード用メッセージIDを取得
	 * @see com.clustercontrol.monitor.run.factory.OperationMonitor#getMessageId(int)
	 */
	@Override
	public String getMessageId(int id) {
		
		if(id == PriorityConstant.TYPE_INFO){
			return MESSAGE_ID_INFO;
		}
		else if(id == PriorityConstant.TYPE_WARNING){
			return MESSAGE_ID_WARNING;
		}
		else if(id == PriorityConstant.TYPE_CRITICAL){
			return MESSAGE_ID_CRITICAL;
		}
		else{
			return MESSAGE_ID_UNKNOWN;
		}
	}
	
	/* (非 Javadoc)
	 * ノード用メッセージを取得
	 * @see com.clustercontrol.monitor.run.factory.OperationMonitor#getMessage(int)
	 */
	@Override
	public String getMessage(int id) {
		return m_message;
	}
	
	/* (非 Javadoc)
	 * ノード用オリジナルメッセージを取得
	 * @see com.clustercontrol.monitor.run.factory.OperationMonitor#getMessageOrg(int)
	 */
	@Override
	public String getMessageOrg(int id) {
		return m_messageOrg;
	}
	
	/* (非 Javadoc)
	 * スコープ用メッセージIDを取得
	 * @see com.clustercontrol.monitor.run.factory.RunMonitor#getMessageIdForScope(int)
	 */
	@Override
	public String getMessageIdForScope(int priority) {
		
		if(priority == PriorityConstant.TYPE_INFO){
			return MESSAGE_ID_INFO;
		}
		else if(priority == PriorityConstant.TYPE_WARNING){
			return MESSAGE_ID_WARNING;
		}
		else if(priority == PriorityConstant.TYPE_CRITICAL){
			return MESSAGE_ID_CRITICAL;
		}
		else{
			return MESSAGE_ID_UNKNOWN;
		}
	}
	
	/**
	 * SNMP収集値の共有テーブルをルックアップ
	 * 
	 */
	private void setSnmpSharedTable() throws NamingException{
		
		// SNMP収集値の共有テーブルをルックアップ
		m_sst = SnmpProcConnectionManager.getConnectionManager().getSnmpSharedTable();
	}
}
